/*
 *     Copyright: Eugene M. Hutorny (c) 2009, eugene@hutorny.in.ua
 *     License:   Licensed to public under The GNU Lesser General Public License (LGPLv3) 
 *                http://www.opensource.org/licenses/lgpl-3.0.html
 */
package ua.in.hutorny.otj.exer1.money;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map.Entry;

import ua.in.hutorny.otj.exer1.exeptions.NotEnoughCash;

public abstract class Treasury {
	private final HashMap<Currency,Cash> cashkeeper = new HashMap<Currency,Cash>();
	
	public final void put(Cash somecash) {
		Cash cash = cashkeeper.get(somecash.currency());
		if( null == cash ) {
			cash = new Cash(somecash.currency());
			cashkeeper.put(cash.currency(),cash);
		}
		cash.add(somecash);
	}
	
	protected Treasury() {
		super();
	}
	
	public final Cash get(BigDecimal amount, Currency currency) throws NotEnoughCash {
		Cash cash = cashkeeper.get(currency);
		if( null == cash) throw new NotEnoughCash(currency.name());
		return cash.subtract(amount);
	}
		
	public final Cash get(double amount, Currency currency) throws NotEnoughCash {
		return get(new BigDecimal(amount), currency);
	}
	public final Cash get(int amount, Currency currency) throws NotEnoughCash {
		return get(new BigDecimal(amount), currency);
	}

	@Override
	public String toString() {
		String result = getClass().getSimpleName() + "(";
		String dlm = "";
		for(Entry<Currency,Cash> i : cashkeeper.entrySet() ) {
			result += dlm + String.format("%s: % 12.2f", i.getValue().currency(), i.getValue().value()); 
			dlm = "; ";			
		}
		return result + ")";
	}

	@Override
	protected void finalize() throws Throwable {
		for(Entry<Currency,Cash> i : cashkeeper.entrySet() ) {
			// money in a treasury are not lost 
			i.getValue().discard();
		}		
		super.finalize();
	}
	
}
